/*
 * yasapi_property.c
 * Copyright (C) 2015-2016 Peter Belkner <pbelkner@snafu.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 */
#include <yasapi.h>

///////////////////////////////////////////////////////////////////////////////
#define PROPERTY_SIZE YASAPI_PROPERTY_SIZE

///////////////////////////////////////////////////////////////////////////////
void PropertiesLoad(const Property *pProperty, PropertyIOConfig *c)
{
  while (pProperty->type) {
    pProperty->type->Load(pProperty,c);
    ++pProperty;
  }
}

void PropertiesSave(const Property *pProperty, const PropertyIOConfig *c)
{
  while (pProperty->type) {
    pProperty->type->Save(pProperty,c);
    ++pProperty;
  }
}

///////////////////////////////////////////////////////////////////////////////
void PropertySaveInt(const wchar_t *group, const wchar_t *key, int n,
    const wchar_t *path)
{
  wchar_t buf[YASAPI_PROPERTY_SIZE];

  swprintf(buf,YASAPI_PROPERTY_SIZE,L"%d",n);

  WritePrivateProfileStringW(
    group,    // _In_  LPCTSTR lpAppName,
    key,      // _In_  LPCTSTR lpKeyName,
    buf,      // _In_  LPCTSTR lpString,
    path      // _In_  LPCTSTR lpFileName
  );
}

static void IntTypeLoad(const Property *pProperty, PropertyIOConfig *c)
{
  PROPERTY_INT(pProperty,c->pData)=GetPrivateProfileIntW(
    c->group,         // _In_  LPCTSTR lpAppName,
    pProperty->key,   // _In_  LPCTSTR lpKeyName,
    PROPERTY_INT(pProperty,c->pDefault),
                      // _In_  INT nDefault,
    c->path           // _In_  LPCTSTR lpFileName
  );

  DWPRINTF(2,L"%soption \"%s\": %d (%d)\n",
      c->pfx,pProperty->key,
      PROPERTY_INT(pProperty,c->pData),
      PROPERTY_INT(pProperty,c->pDefault));
}

static void IntTypeSave(const Property *pProperty, const PropertyIOConfig *c)
{
#if 0
  wchar_t buf[PROPERTY_SIZE];

  swprintf(buf,PROPERTY_SIZE,L"%d",PROPERTY_INT(pProperty,c->pData));

  WritePrivateProfileStringW(
    c->group,         // _In_  LPCTSTR lpAppName,
    pProperty->key,   // _In_  LPCTSTR lpKeyName,
    buf,              // _In_  LPCTSTR lpString,
    c->path           // _In_  LPCTSTR lpFileName
  );
#else 
  PropertySaveInt(c->group,pProperty->key,PROPERTY_INT(pProperty,c->pData),
      c->path);
#endif
}

const PropertyType gcIntType={
  IntTypeLoad,
  IntTypeSave
};

///////////////////////////////////////////////////////////////////////////////
static void DoubleTypeLoad(const Property *pProperty, PropertyIOConfig *c)
{
  wchar_t set[PROPERTY_SIZE];
  wchar_t get[PROPERTY_SIZE];

  swprintf(set,PROPERTY_SIZE,L"%f",PROPERTY_DOUBLE(pProperty,c->pDefault));

  GetPrivateProfileStringW(
    c->group,         // _In_   LPCTSTR lpAppName,
    pProperty->key,   // _In_   LPCTSTR lpKeyName,
    set,              // _In_   LPCTSTR lpDefault,
    get,              // _Out_  LPTSTR lpReturnedString,
    PROPERTY_SIZE,    // _In_   DWORD nSize,
    c->path           // _In_   LPCTSTR lpFileName
  );

  PROPERTY_DOUBLE(pProperty,c->pData)=_wtof(get);
  DWPRINTF(2,L"%soption \"%s\": %f (%f)\n",
      c->pfx,pProperty->key,
      PROPERTY_DOUBLE(pProperty,c->pData),
      PROPERTY_DOUBLE(pProperty,c->pDefault));
}

static void DoubleTypeSave(const Property *pProperty,
    const PropertyIOConfig *c)
{
  wchar_t buf[PROPERTY_SIZE];

  swprintf(buf,PROPERTY_SIZE,L"%f",PROPERTY_DOUBLE(pProperty,c->pData));

  WritePrivateProfileStringW(
    c->group,         // _In_  LPCTSTR lpAppName,
    pProperty->key,   // _In_  LPCTSTR lpKeyName,
    buf,              // _In_  LPCTSTR lpString,
    c->path           // _In_  LPCTSTR lpFileName
  );
}

const PropertyType gcDoubleType={
  DoubleTypeLoad,
  DoubleTypeSave
};

///////////////////////////////////////////////////////////////////////////////
static void StringTypeLoad(const Property *pProperty, PropertyIOConfig *c)
{
  GetPrivateProfileStringW(
    c->group,         // _In_   LPCTSTR lpAppName,
    pProperty->key,   // _In_   LPCTSTR lpKeyName,
    PROPERTY_STRING(pProperty,c->pDefault),
                      // _In_   LPCTSTR lpDefault,
    PROPERTY_STRING(pProperty,c->pData),
                      // _Out_  LPTSTR lpReturnedString,
    pProperty->size,  // _In_   DWORD nSize,
    c->path           // _In_   LPCTSTR lpFileName
  );

  DWPRINTF(2,L"%soption \"%s\": \"%s\" (\"%s\")\n",
      c->pfx,pProperty->key,
      PROPERTY_STRING(pProperty,c->pData),
      PROPERTY_STRING(pProperty,c->pDefault));
}

static void StringTypeSave(const Property *pProperty,
    const PropertyIOConfig *c)
{
  WritePrivateProfileStringW(
    c->group,         // _In_  LPCTSTR lpAppName,
    pProperty->key,   // _In_  LPCTSTR lpKeyName,
    PROPERTY_STRING(pProperty,c->pData),
                      // _In_  LPCTSTR lpString,
    c->path           // _In_  LPCTSTR lpFileName
  );
}

const PropertyType gcStringType={
  StringTypeLoad,
  StringTypeSave
};
